"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
function typescriptTransformUnspec(program) {
    return (context) => (file) => visitNodeAndChildren(file, program, context);
}
exports.default = typescriptTransformUnspec;
function visitNodeAndChildren(node, program, context) {
    const visitor = (childNode) => visitNodeAndChildren(childNode, program, context);
    return ts.visitEachChild(filterNode(node), visitor, context);
}
function filterNode(node) {
    if (isSpecExpressionStatement(node)) {
        return undefined;
    }
    return node;
}
// todo: delete only global
const testMethodList = [
    'jasmine',
    'jest',
    'before',
    'after',
    'beforeAll',
    'beforeEach',
    'afterAll',
    'afterEach',
    'describe',
    'fdescribe',
    'xdescribe',
    'it',
    'fit',
    'xit',
    'test',
    'xtest',
    'spyOn',
];
function isTestExpressionText(text) {
    return testMethodList.includes(String(text));
}
function isSpecExpressionStatement(node) {
    if (ts.isExpressionStatement(node) && ts.isCallExpression(node.expression)) {
        const callExpr = node.expression.expression;
        if (ts.isIdentifier(callExpr) && isTestExpressionText(callExpr.escapedText)) {
            return true;
        }
        if (ts.isPropertyAccessExpression(callExpr)) {
            const propertyAccessExpr = callExpr.expression;
            if (ts.isIdentifier(propertyAccessExpr) && isTestExpressionText(propertyAccessExpr.escapedText)) {
                return true;
            }
        }
    }
    return false;
}
module.exports = typescriptTransformUnspec;
module.exports.default = typescriptTransformUnspec;
//# sourceMappingURL=index.js.map